/*
Der Original-Code von Peter Dannegger vom ATmega644 eines  8-bit Timer/Counter0
wird auf einen TCA - 16-bit Timer/Counter eines ATTiny1627 uebertragen.
Master-Rolle. Es wird der Pin PB3 als i/o genutzt.
*/

void tx_data( uint8_t addr, uint8_t data) 
{
	PORTB_PIN3CTRL = 0;			//*erl* PCMSK1 = 0;   // disable receive 				
	TCA0_SINGLE_CTRLA = 0;	// stop T0
	tx_idx = 0;          	// point to first bit
	tx_buff[0] = addr;
	tx_buff[1] = data;
	tx_buff[2] = Slow_CRC_Cal8Bits(0x00, 2, tx_buff);

	PORTB.DIRCLR = PIN3_bm;			// input 
	PORTB_PIN3CTRL = (1<<3);		//pullup PB4 
	while( (PORTB.IN & PIN3_bm) == 0 );// wait until pin high
	TCA0_SINGLE_CNT = 0;
//	OCR0A = BIT_TIME * 4 - 0.5;   // sync: 4T The OCRA contains an 8-bit value that is continuously compared with the counter value (TCNT0)
	TCA0_SINGLE_CMP0 = BIT_TIME * 4 - 0.5;
	TCA0_SINGLE_CTRLB = TCA_SINGLE_CMP0EN_bm | 1;  // Compare0 Enable	
	PORTMUX_TCAROUTEA = PORTMUX_TCA00_bm;  		// OC auf Alternative Pin PB3 legen.
	TCA0_SINGLE_CTRLC = TCA_SINGLE_CMP0OV_bm; 	// Compare-Ausgang auf 1 setzen.
	PORTB_DIRSET = PIN3_bm;			// The direction for the associated port pin n must be configured in the Port peripheral as an output			
	TCA0_SINGLE_INTFLAGS = TCA_SINGLE_CMP0_bm; // clear earlier interrupts
	TCA0_SINGLE_INTCTRL = TCA_SINGLE_CMP0EN_bm; // enable tx interrupt
	TCA0_SINGLE_CTRLA = TCA_SINGLE_CLKSEL_DIV64_gc | TCA_SINGLE_ENABLE_bm; // Vorteiler, TCA Enable/
//	while( TCA0_SINGLE_CTRLA );  // untile tx done
}

ISR (TCA0_CMP0_vect)
{
volatile uint8_t *dp;					// locals for faster access
if( tx_idx < DATA_SIZE * 8 )
	{TCA0_SINGLE_CMP0 = BIT_TIME * 1 - 0.5;     // 0-bit: 1T
    dp = &tx_buff[tx_idx++ / 8];                // get byte address
    if( *dp & 0x80 )                            // msb first
		TCA0_SINGLE_CMP0 = BIT_TIME * 2 - 0.5;  // 1-bit: 2T
    *dp <<= 1;                                  // shift bits
	}
else{
    TCA0_SINGLE_CMP0 = BIT_TIME * 4 - 0.5;      // sync:  4T
    if( PORTB.IN & PIN3_bm)                     // "DATA_PIN" after output = high
		TCA0_SINGLE_CTRLA = 0; //TCCR0B = 0;    // then stop T0
	}	
TCA0_SINGLE_INTFLAGS = TCA_SINGLE_CMP0_bm;
}

ISR ( TCA0_CMP1_vect )         // sync received
{
if( rx_idx == DATA_SIZE * 8 )  // receive successful
	{
	PORTB_PIN3CTRL &= ~PORT_ISC_gm; // PCMSK1 = 0;// disable further receive
	task |= (1<<taskrxready);
	}
rx_idx = 255;               // detect sync transition
if( rx_timeout )
	{						// timeout answer from slave
	rx_timeout--;
	TCA0_SINGLE_CNT = 0; //TCNT0 = 0;
	}
else {TCA0_SINGLE_INTCTRL &= ~((TCA_SINGLE_CMP0EN_bm)|(TCA_SINGLE_CMP1EN_bm));	
	 }
TCA0_SINGLE_INTFLAGS = TCA_SINGLE_CMP1_bm;
}

void rx_init( void )
{
TCA0_SINGLE_CTRLA = 0; 		//TCCR0B = 0;	// Stop T0,  No clock source (Timer/Counter stopped)
rx_timeout = 10;			// 28 * T
rx_idx = 255;        		// detect sync transition
PORTB.DIRCLR = PIN3_bm; 	//	DATA_DDR = 0;                 // input
PORTB_PIN3CTRL = (1<<3);	//DATA = 1;  pullup on
TCA0_SINGLE_CTRLB = TCA_SINGLE_CMP1EN_bm | TCA_SINGLE_CMP0EN_bm;
TCA0_SINGLE_CNT = 0; 	// Counter auf 0
TCA0_SINGLE_CMP0 = BIT_TIME * 1.4 - 0.5;  // detect 2T 
TCA0_SINGLE_CMP1 = BIT_TIME * 2.8 - 0.5; // detect 4T
TCA0_SINGLE_INTFLAGS = TCA_SINGLE_CMP0_bm |TCA_SINGLE_CMP1_bm; // Interrupt flags loeschen
TCA0_SINGLE_INTCTRL &= ~((TCA_SINGLE_CMP0EN_bm)|(TCA_SINGLE_CMP1EN_bm)); // noch keine Timer Interrupts nutzen
TCA0_SINGLE_CTRLA = TCA_SINGLE_CLKSEL_DIV64_gc | TCA_SINGLE_ENABLE_bm;	// Zaehler starten
PORTB.INTFLAGS &= PIN3_bm; 	// Portpin-Interrupt flag loeschen
PORTB_PIN3CTRL |= 1; 		//  pin change interrupt 
}

ISR (PORTB_PORT_vect)
{
/* anderer PORTB Interrupt code */
	
if (PORTB.INTFLAGS & PIN3_bm) //ISR( PCINT1_vect )              // Receive interrupt
	{
	uint8_t *dp;					// locals for faster access
	uint8_t i;                    // locals for faster access

	TCA0_SINGLE_CNT = 0; //TCNT0 = 0;
	if( rx_idx >= DATA_SIZE * 8 ){ // on start or stop bit
		rx_idx = 0;  }               // point to first bit
	else{
		dp = &rx_buff[rx_idx / 8];   // get byte address
		i = *dp << 1;                // shift bits
//		if( TIFR0 & 1<<OCF0A )       // if pulse > 1.4T
		if (TCA0_SINGLE_INTFLAGS & TCA_SINGLE_CMP0_bm)
		i++;                       // msb first
		*dp = i;
		rx_idx++;                    // point to next bit
		}
	TCA0_SINGLE_INTFLAGS = TCA_SINGLE_CMP0_bm | TCA_SINGLE_CMP1_bm ; //clear 2T & 4T detection flag
	TCA0_SINGLE_INTCTRL = TCA_SINGLE_CMP1EN_bm;
	PORTB.INTFLAGS &= PIN3_bm;
	}
}
